using Engine;
using Engine.Input;
using Serilog.Events;

namespace PresenterNative;

internal static class Program
{
  public static void Main(string[] parArgs)
  {
    var engine = new EngineBuilder()
      .Headless(false)
      .Width(1280)
      .Height(720)
      .Title("Doom Deathmatch")
      .LogToConsole()
      .LogToFile(true, "log.txt")
      .LogLevel(LogEventLevel.Debug)
      .Presenter(parEngine => parEngine.Window)
      .InputHandler(parEngine => new WindowInputHandler(parEngine.Window))
      .AssetFolder(Path.GetFullPath("../DoomDeathmatch/asset"))
      .Build();

    DoomDeathmatch.DoomDeathmatch.Initialize(engine);

    engine.Run();
  }
}